<?php

namespace Drupal\uber_affiliate;

use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Component\Utility\Xss;
use Drupal\Component\Utility\StringHelper;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Database\Query\PagerSelectExtender;
use Drupal\Core\Database\Query\TableSortExtender;

trait UberAffiliateTrait
{
  use DependencySerializationTrait;

  /**
   * The Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The Date Formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * Sets the entity type manager.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function setEntityTypeManager(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Sets the Date Formatter service.
   *
   * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   The Date Formatter service.
   */
  public function setDateFormatter(DateFormatter $dateFormatter) {
    $this->dateFormatter = $dateFormatter;
  }

  /**
   * Gets the Date Formatter service.
   *
   * @return \Drupal\Core\Datetime\DateFormatter
   *   The Date Formatter service.
   */
  public function getDateFormatter() {
    return $this->dateFormatter;
  }

  /**
   * Gets the entity type manager.
   *
   * @return \Drupal\Core\Entity\EntityTypeManagerInterface
   *   The entity type manager.
   */
  public function getEntityTypeManager() {
    return $this->entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $entityTypeManager = $container->get('entity_type.manager');
    $dateFormatter = $container->get('date.formatter');


    $instance = new static();
    $instance->setEntityTypeManager($entityTypeManager);
    $instance->setDateFormatter($dateFormatter);


    return $instance;
  }

  /**
   * Fetch an affiliate object.
   *
   * @param int $affid
   *   Affiliate ID (users.uid)
   *
   * @return object
   *   If affiliate exists, affiliate info (uid, active, created, clicks, referrals,
   *   payouts_owed, payouts_paid). Otherwise, empty.
   *
   */
  public function affiliate_load_affiliate($affid)
  {
    $affid = (int)$affid;
    $res = \Drupal::database()->query("SELECT * FROM {affiliate} WHERE {affiliate}.uid = :affid", array(":affid" => $affid));
    $affiliate = $res->fetchObject();
    return $affiliate;
  }

  /**
   * Add a new affiliate.
   *
   * @param int $uid
   *   User ID of the affiliate being created.
   *
   * @param bool $status
   *   (Optional) If affiliate is to be active, set $status to 1 (default). Otherwise,
   *   setting $status to 0 will still create the affiliate but the affiliate will not be active.
   *
   */
  public function affiliate_insert_affiliate($uid, $status = 1)
  {
    $uid = (int)$uid;
    $active = 0;
    if ($status) {
      $active = 1;
    }
    $time = time();
    \Drupal::database()->insert('affiliate')
      ->fields(array(
        'uid' => $uid,
        'active' => $active,
        'created' => $time,
      ))
      ->execute();
  }

  /**
   * Update an affiliate's status.
   *
   * @param int $uid
   *   User ID of the affiliate whose status is being changed.
   *
   * @param bool $status
   *   1 = Active, 0 = Inactive
   */
  public function affiliate_set_affiliate_status($uid, $status)
  {
    $uid = (int)$uid;
    $active = 0;
    if ($status) {
      $active = 1;
    }
    \Drupal::database()->update('affiliate')
      ->condition('uid', $uid)
      ->fields(array(
        'active' => $active,
      ))
      ->execute();
  }



  /**
   * Attempt to make sure an incoming click was made validly.
   *
   * @param int $affid
   *   Affiliate (user) ID of the affiliate who should receive the credit for this click.
   *
   * @param string $dest
   *   The destination, formatted as a relative path, that the user should be redirected to upon validation.
   *
   * @param bool $verify_node_type
   *   (Optional) Verify that the node type to which the user is being redirected is one
   *   of the types allowed as set at admin/config/people/affiliate/content.
   *
   */
  public function affiliate_validate_click($affid, $dest, $verify_node_type = TRUE)
  {

    $affid = (int)$affid;
    $dest = UrlHelper::stripDangerousProtocols($dest);

    $node_path_pattern = '<^(node/)([0-9]+)$>';
    $is_node_path = preg_match($node_path_pattern, $dest, $node_path_matches);
    $nid = 0;
    if (isset($node_path_matches) && !empty($node_path_matches) && isset($node_path_matches[2]) && $node_path_matches[2] > 0) {
      // filter_xss() necessary?
      $nid = (int)Xss::filter($node_path_matches[2]);
    }
    $allow_all_types = \Drupal::state()->get('affiliate_module_allow_all_node_types', 0);
    $allow_all_paths = \Drupal::state()->get('affiliate_module_allow_all_paths', 0);
    if ($allow_all_paths) {
      if (\Drupal::service('path.validator')->isValid($dest) && !UrlHelper::isExternal($dest)) {
        return TRUE;
      }
    }

    if ($allow_all_types || $verify_node_type) {
      if ($is_node_path && $nid) {
        if ($allow_all_types && \Drupal::service('path.validator')->isValid($dest) && !url_is_external($dest)) {
          return TRUE;
        }
        if ($verify_node_type) {
          $node = \Drupal::service('entity_type.manager')->getStorage('node')->load($nid);
          $type = $node->getType();
          $allowed_node_types = \Drupal::state()->get('affiliate_module_allowed_node_types', ['article']);
          if (!in_array($type, (array)$allowed_node_types)) {
            return FALSE;
          }
        }
      }
    }
    if (!affiliate_valid_destination($dest)) {
      return FALSE;
    }
    if (!affiliate_referrer_exists()) {
      return FALSE;
    }
    if (!affiliate_validate_uid($affid)) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Verify that an affiliate destination path exists, is not external, and is not administrative in nature.
   *
   * @param string $dest
   *   The destination, formatted as a relative path, that the user should be redirected to upon validation.
   *
   * @return bool
   *   TRUE if $dest is a valid affiliate path, otherwise FALSE.
   */
  public function affiliate_valid_destination($dest)
  {
    if (!\Drupal::service('path.validator')->isValid($dest)) {
      return FALSE;
    }
    if (UrlHelper::isExternal($dest)) {
      return FALSE;
    }
    //if (\Drupal::service('router.admin_context')->isAdminRoute($dest)) {
    //}
    return TRUE;
  }


  /**
   * Check to see if the $_SERVER['HTTP_REFERER'] requirements, if any, have been met,
   * insofar as existence is concerned.
   *
   * @return bool
   *   FALSE if referral source is not set, provided the system has not been told to ignore
   *   this setting. Otherwise TRUE.
   */
  public function affiliate_referrer_exists()
  {
    if (\Drupal::state()->get('affiliate_module_check_referrer', 1)) {
      if (!isset($_SERVER['HTTP_REFERER']) || empty($_SERVER['HTTP_REFERER'])) {
        return FALSE;
      }
    }
    return TRUE;
  }

  /**
   * Alias function for people who might forget that REFERER is not the correct spelling of
   * the word REFERRER. ;)
   */
  public function affiliate_referer_exists()
  {
    return affiliate_referrer_exists();
  }

  /**
   * Generate a path for each affiliate to use based on the current path and their user id.
   *
   * @param int $uid
   *   The {users}.uid of the affiliate. If $uid is not provided or is set to NULL, the
   *   value will default to the current user's {users}.uid.
   *
   * @return string
   *   A path which can be used to generate a clickable URL with which an affiliate can
   *   receive credit.
   *
   * @see affiliate_fetch_url()
   * @see affiliate_validate_click()
   * @see affiliate_node_markup()
   */
  function affiliate_fetch_path($uid = NULL)
  {
    if (!empty($uid) && !is_null($uid)) {
      $user_id = $uid;
    } else {
      $user_id = $GLOBALS['user']->uid;
    }
    $user_id = (int)$user_id;
    $current_path = current_path();
    $dest_path = str_replace('/', '|', $current_path);
    $menu_prefix = \Drupal::state()->get('affiliate_module_affiliate_menu_path', 'affiliate');
    $affiliate_path = $menu_prefix . '/' . $user_id . '/' . $dest_path;
    $out = check_url($affiliate_path);
    return $out;
  }

  /**
   * Generate a url for each affiliate to use based on the current path and their user id.
   *
   * @param int $uid
   *   The {users}.uid of the affiliate. If $uid is not provided or is set to NULL, the
   *   value will default to the current user's {users}.uid.
   *
   * @return string
   *   A URL which, when clicked by an outside party, will give the affiliate credit for
   *   a click-thru.
   *
   * @see affiliate_fetch_path()
   * @see affiliate_validate_click()
   * @see affiliate_node_markup()
   */
  public function affiliate_fetch_url($uid = NULL)
  {
    global $base_url;
    if (!empty($uid) && !is_null($uid)) {
      $user_id = $uid;
    } else {
      $user_id = $GLOBALS['user']->uid;
    }
    $user_id = (int)$user_id;
    $path = affiliate_fetch_path($user_id);
    $url_raw = $base_url . base_path() . $path;
    $url = check_url($url_raw);
    $out = url($url);
    return $out;
  }

  /**
   * Attempts to fetch a campaign ID based on the node nid. Will not always work when, for
   * instance, all paths are allowed as affiliate paths.
   *
   * @return
   *   {node}.nid if found, otherwise 0.
   */
  public function affiliate_get_campaign_id_by_nid($nid)
  {
    $nid = (int)$nid;
    $cid = \Drupal::database()->query("SELECT cid FROM {affiliate_campaigns} WHERE eid = :nid AND etype = 'node'", array(":nid" => $nid))->fetchField();
    $out = ($cid && !empty($cid)) ? (int)$cid : 0;
    return $out;
  }

  /**
   * Attempts to fetch a campaign ID based on the visited path. Will not always work when, for
   * instance, all paths are allowed as affiliate paths.
   *
   * @return
   *   Campaign ID if found, otherwise 0.
   */
  public function affiliate_get_campaign_id($path)
  {
    $path = check_url(rawurldecode($path));
    $cid = \Drupal::database()->query("SELECT ac.cid FROM {affiliate_campaigns} ac WHERE ac.redirect_path = :path", array(":path" => $path))->fetchField();
    $out = ($cid && !empty($cid)) ? (int)$cid : 0;
    return $out;
  }

  /**
   * Record a click-thru, giving credit to the specified affiliate ID.
   *
   * @param int $affid
   *   The {affiliate}.uid of the affiliate
   *
   * @param int $campaign_id
   *   The {affiliate_campaigns}.cid of the specified campaign, if any.
   *
   * @return void
   *   Credit for the click applied to the affiliate account if validation attempts are passed.
   *
   * @see affiliate_click_denied()
   */
  public function affiliate_add_click($affid, $campaign_id = 0)
  {
    $cookie_value = md5(microtime());
    $timeout_flag = true;
    $affid = (int)$affid;
    $campaign_id = (int)$campaign_id;
    $time = time();
    if ($affid) {
      $affiliate_click_denied = affiliate_click_denied($affid);
      if ($_COOKIE && array_key_exists('AFFILIATE', $_COOKIE) && !$_COOKIE['AFFILIATE']) {
        setcookie('AFFILIATE', $cookie_value, \Drupal::time()->getRequestTime() + 86400);
      }
      // Only record the click-thru if it looks like this isn't a cheater and
      // if the user who clicked has the permission to have her click counted.
      if (!$affiliate_click_denied && user_access('track affiliate clicks for this role')) {
        \Drupal::database()->insert('affiliate_clicks')
          ->fields(array(
            'uid' => $affid,
            'cid' => $campaign_id,
            'click_time' => $time,
            'cookie_id' => $cookie_value,
            'ip' => ip_address(),
            'referrer' => $_SERVER['HTTP_REFERER'],
          ))
          ->execute();
        // If the Userpoints module exists, add points as configured at admin/config/people/affiliate/payouts
        if (\Drupal::moduleHandler()->moduleExists('userpoints')) {
          $points = (int)\Drupal::state()->get('affiliate_module_payouts_userpoints_clicks', 0);
          $params = array(
            'points' => $points,
            'uid' => $affid,
            'operation' => 'affiliate click',
            'tid' => \Drupal::state()->get('affiliate_module_payouts_userpoints_tid', 0),
            'reference' => 'affiliate'
          );
          userpoints_userpointsapi($params);
        }
        // Allow the Referral module to set its cookie
        if (\Drupal::moduleHandler()->moduleExists('referral')) {
          referral_set_cookie($affid);
        }
      }
    }
  }

  public function affiliate_validate_uid($affid = 0)
  {
    global $user;
    $affid = (int)$affid;
    $uid = (int)$user->uid;
    $valid = TRUE;
    // Fail if affiliate ID from the URL is the same as the currently logged-in user ID.
    if ($affid == $uid) {
      $valid = FALSE;
    }
    // Fail if affiliate ID is not set or is set to 0 (which, essentially, means that it's not set).
    if (!isset($affid) || empty($affid) || $affid < 1) {
      $valid = FALSE;
    }
    return $valid;
  }

  public function affiliate_click_denied($uid)
  {
    $account = \Drupal::service('entity_type.manager')->getStorage('user')->load($uid);
    $denied_users = explode("\n", \Drupal::state()->get('affiliate_module_ignored_users', ''));
    if (in_array($account->name, $denied_users)) {
      return TRUE;
    }
    $ip = ip_address();
    $denied_ips = explode("\n", \Drupal::state()->get('affiliate_module_ignored_ips', ''));
    if (in_array($ip, $denied_ips)) {
      return TRUE;
    }
    if (!affiliate_referrer_exists()) {
      return TRUE;
    }
    $too_soon = _affiliate_check_timer();
    if ($too_soon) {
      return TRUE;
    }
    $denied_uri_referrers = explode("\n", \Drupal::state()->get('affiliate_module_ignored_uri_referrers', ''));
    if (count($denied_uri_referrers)) {
      foreach ($denied_uri_referrers as $referrer) {
        if (!empty($referrer) && strpos($_SERVER['HTTP_REFERER'], $referrer)) {
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  public function _affiliate_check_timer()
  {
    $enabled = \Drupal::state()->get('affiliate_module_click_ignore_enable', 0);
    if (!$enabled) {
      return FALSE;
    }
    $ip = ip_address();
    // Default time limit allows for one click per IP per day.
    $time = time();
    $interval = \Drupal::state()->get('affiliate_module_click_ignore_interval', 86400);
    $limit = $time - $interval;
    return (bool)\Drupal::database()->query("SELECT COUNT(*) FROM {affiliate_clicks} WHERE ip = :ip AND click_time > :limit", array(":ip" => $ip, ":limit" => $limit))->fetchField();
  }


  public function affiliate_admin_overviewpage()
  {
    $out = affiliate_admin_overviewpage_output();
    $css = '.affiliate-payout-inlineform-amount, .affiliate-payout-inlineform-submit {display: inline-block;}';
    drupal_add_css($css, 'inline');
    drupal_set_title(t('Affiliate management: overview'));
    return $out;
  }

  public function affiliateUserPageOutput($acctid)
  {
    $out = $this->affiliate_user_page_output_top($acctid);
    $out += $this->affiliate_user_page_output_details($acctid);
    return $out;
  }

  public function affiliate_user_page_output_details($acctid)
  {
    $header = array(
      array('data' => t('Date'), 'field' => 'click_time', 'sort' => 'desc'),
      array('data' => t('Referrer'), 'field' => 'referrer'),
    );

    $count_query = \Drupal::database()->select('affiliate_clicks')
      ->condition('uid', $acctid);
    $count_query->addExpression('COUNT(DISTINCT(clickid))');

    $query = \Drupal::database()->select('affiliate_clicks', 'a')->extend(PagerSelectExtender::class)->extend(TableSortExtender::class)
      ->condition('uid', $acctid)
      ->fields('a', array('click_time', 'referrer'))
      ->limit(30)
      ->orderByHeader($header);
    $query->setCountQuery($count_query);
    $result = $query->execute();

    $rows = array();
    foreach ($result as $res) {
      $click_time = \Drupal::service('date.formatter')->format($res->click_time);
      $referrer = url($res->referrer);
      $rows[] = array($click_time, $referrer);
    }

    $build['affiliate_bottom_table'] = array(
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => t('No click information available.'),
    );

    return $build;
  }

  public function affiliate_user_page_output_top($acctid)
  {
    $header = array(
      array('data' => t('Active')),
      array('data' => t('Affiliate since')),
      array('data' => t('Clicks')),
      array('data' => t('Referrals')),
      array('data' => t('Payouts owed')),
      array('data' => t('Payouts paid'))
    );
    $payouts_symbol = \Drupal::state()->get('affiliate_module_payouts_symbol', '$');
    $payouts_symbol_pos = \Drupal::state()->get('affiliate_module_payouts_symbol_placement', 1);
    $payouts_amount = \Drupal::state()->get('affiliate_module_payouts_per_click', 0.00);

    $query = \Drupal::database()->select('affiliate', 'a')
      ->condition('uid', $acctid)
      ->fields('a', array('uid', 'active', 'created', 'clicks', 'referrals', 'payouts_owed', 'payouts_paid'));
    $result = $query->execute();

    $rows = array();
    foreach ($result as $affiliate) {
      $affiliate_uid = $affiliate->uid;
      $affiliate_name = $this->entityTypeManager->getStorage('user')->load($affiliate_uid)->getDisplayName();
      $affiliate_uid_display = Link::fromTextAndUrl(
        Html::escape('@uid - @name', ['@uid' => $affiliate_uid, '@name' => $affiliate_name]),
        Url::fromRoute('entity.user.canonical', ['user' => $affiliate_uid])
      )->toString();
      $affiliate_active = $affiliate->active ? t('Yes') : t('No');
      $affiliate_created = $this->dateFormatter->format($affiliate->created);
      $payouts_owed_display = $payouts_symbol . $affiliate->payouts_owed;
      $payouts_paid_display = $payouts_symbol . $affiliate->payouts_paid;
      if ($payouts_symbol_pos == 2) {
        $payouts_owed_display = $affiliate->payouts_owed . $payouts_symbol;
        $payouts_paid_display = $affiliate->payouts_paid . $payouts_symbol;
      }
      $rows[] = [
        $affiliate_active,
        $affiliate_created,
        $affiliate->clicks,
        $affiliate->referrals,
        $payouts_owed_display,
        $payouts_paid_display,
      ];
    }

    $build['affiliate_top_table'] = array(
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => t('No affiliate information available.'),
    );

    return $build;
  }

  public function affiliate_admin_overviewpage_output()
  {
    $header = array(
      array('data' => t('ID'), 'field' => 'uid'),
      array('data' => t('Name'), 'field' => 'name'),
      array('data' => t('Active'), 'field' => 'active'),
      array('data' => t('Created'), 'field' => 'created', 'sort' => 'desc'),
      array('data' => t('Clicks'), 'field' => 'clicks'),
      array('data' => t('Referrals'), 'field' => 'referrals'),
      array('data' => t('Payouts owed'), 'field' => 'payouts_owed'),
      array('data' => t('Payouts paid'), 'field' => 'payouts_paid'),
      array('data' => '', 'field' => NULL),
    );
    $payouts_symbol = \Drupal::state()->get('affiliate_module_payouts_symbol', '$');
    $payouts_symbol_pos = \Drupal::state()->get('affiliate_module_payouts_symbol_placement', 1);
    $payouts_amount = \Drupal::state()->get('affiliate_module_payouts_per_click', 0.00);
    $count_query = \Drupal::database()->query('affiliate');
    $count_query->addExpression('COUNT(DISTINCT(uid))');

    $query = \Drupal::database()->query('affiliate', 'a')->extend('PagerDefault')->extend('TableSort');
    $query->innerjoin('users', 'u', 'a.uid = u.uid');
    $query = $query
      ->fields('a', array('uid', 'active', 'created', 'clicks', 'referrals', 'payouts_owed', 'payouts_paid'))
      ->fields('u', array('name'))
      ->groupBy('a.uid')
      ->limit(30)
      ->orderByHeader($header);
    $query->setCountQuery($count_query);
    $result = $query->execute();

    $rows = array();
    foreach ($result as $affiliate) {
      $affiliate_uid = $affiliate->uid;
      $affiliate_name = check_plain($affiliate->name);
      $affiliate_uid_display = l($affiliate_uid, drupal_get_path_alias('user/' . $affiliate_uid));
      $affiliate_name_display = l($affiliate_name, drupal_get_path_alias('user/' . $affiliate_uid));
      $affiliate_active = $affiliate->active ? t('Yes') : t('No');
      $affiliate_created = \Drupal::service('date.formatter')->format($affiliate->created);
      $payouts_owed_display = $payouts_symbol . $affiliate->payouts_owed;
      $payouts_paid_display = $payouts_symbol . $affiliate->payouts_paid;
      $payouts_form = '';
      if ($payouts_symbol_pos == 2) {
        $payouts_owed_display = $affiliate->payouts_owed . $payouts_symbol;
        $payouts_paid_display = $affiliate->payouts_paid . $payouts_symbol;
      }
      if (user_access('administer affiliate settings') || user_access('administer affiliate payouts')) {
        $payouts_form .= \Drupal::service('renderer')->render(drupal_get_form('affiliate_payout_inlineform', $affiliate_uid));
      }
      $rows[] = array($affiliate_uid_display, $affiliate_name_display, $affiliate_active, $affiliate_created, $affiliate->clicks, $affiliate->referrals, $payouts_owed_display, $payouts_paid_display, $payouts_form);
    }

    $build['affiliate_top_table'] = array(
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => t('No affiliate information available.'),
    );
    $build['affiliate_top_pager'] = array('#theme' => 'pager');

    return $build;
  }


  /**
   * Top Users Page
   *
   * @return mixed
   */
  public function affiliate_top_users_page()
  {
    $limit = \Drupal::state()->get('affiliate_module_top_users_count_page', 10);
    $time = time();
    $time_interval = \Drupal::state()->get('affiliate_module_top_users_period_interval', 259200);
    $time_limit = $time - $time_interval;
    $results = \Drupal::database()->query("SELECT COUNT(DISTINCT(ac.clickid)) AS clicks, u.uid AS uid, u.name AS name, u.data AS data, aa.payouts_owed AS payouts_owed FROM {affiliate_clicks} ac INNER JOIN {users} u ON ac.uid = u.uid INNER JOIN {affiliate} aa ON u.uid = aa.uid WHERE aa.active = 1 AND ac.click_time > :time_limit GROUP BY uid ORDER BY clicks DESC", array(":time_limit" => $time_limit));
    $rows = array();
    $rank = 1;
    if ($results && !empty($results)) {
      foreach ($results as $key => $result) {
        $row_class = ($key % 2) ? 'affiliate-row-even' : 'affiliate-row-odd';
        $userdata = unserialize($result->data);
        $homepage = (isset($userdata['affiliate_homepage']) && !empty($userdata['affiliate_homepage'])) ? check_url($userdata['affiliate_homepage']) : '';
        $username = check_plain($result->name);
        $name = (!empty($homepage)) ? l($username, $homepage) : l($username, drupal_get_path_alias('user/' . $result->uid));
        $rows[] = array(
          array('class' => array($row_class, 'affiliate-row-rank'), 'data' => $rank++),
          array('class' => array($row_class, 'affiliate-row-name'), 'data' => $name),
          array('class' => array($row_class, 'affiliate-row-clicks'), 'data' => $result->clicks),
          array('class' => array($row_class, 'affiliate-row-payouts-owed'), 'data' => $result->payouts_owed),
        );
      }
    }
    if (empty($rows)) {
      $rows[] = array(array('data' => t('No affiliate activity found for this time period.'), 'colspan' => '4'));
    }
    $header = array('Rank', 'User', 'Clicks', 'Payouts');
    $output = theme('table', array('header' => $header, 'rows' => $rows));
    drupal_set_title(t('Top affiliates from @from through @to', array('@from' => date('Y-m-d', $time_limit), '@to' => date('Y-m-d', $time))));
    return $output;
  }

}
